// Copyright (c) 2002-2007 Telelogic AB
// All rights reserved.

/*
<trigger>
	<usage>bsf transition_associated_tasks.js to_state</usage>
	<version>5.0</version>
	<description>
		CR の遷移中にタスクを指定した状態に関連付けます。
	</description>
	<argument1>
		<name>to_state</name>
		<required>必須</required>
		<type>string</type>
		<description>
			関連タスクの遷移先の状態名。
		</description>
	</argument1>
	<argument2>
		<name>-ifstate state_name</name>
		<required>オプション</required>
		<type>string</type>
		<description>
			この状態のタスクのみが遷移します。
			他の状態のタスクはそのままです。
		</description>
	</argument2>

</trigger>
*/
importPackage(Packages.com.telelogic.cs.api);

var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

// extract the -options
var options = OptionParser.parse(args);

if (options.args.length != 1)
{
	log.error('Could not transition associated tasks: no state was given.');
	log.error('Usage: transition_associated_tasks.js to_state [-ifstate current_state]');
}
else
{
	var toState = options.args[0];
	var expectedState = options.getOption('-ifstate');
	
	// if we need to compare the state, lookup the 'status' too
	var attributeNames = expectedState ? ['status', 'from_db_path'] : ['from_db_path'];
	var tasks = change.getAssociatedTasks(event.triggerUser.token, event.objectId, attributeNames);

	var doneTasks = {};
	var tasksToErrors = {};
	
	for (var i = 0; i < tasks.length; i++) 
	{
		var currentTask = tasks[i];

		if (expectedState)
		{
			var actualState = currentTask.getAttribute('status').stringValue;
			
			// skip any tasks that don't meet the extra criteria of being in a particular state
			if (!expectedState.equals(actualState)) continue;
		}
		
		if (doneTasks[currentTask.taskId]) continue;

		try
		{
			// in central server mode, passing in a from_db_path will override the context database.
			// This lets up modify the task where ever it came from.
			var from = currentTask.getAttribute('from_db_path');
			var newAttributes = from ?
				[new Attribute('from_db_path', from.stringValue)] :
				[];

			change.transitionTask(event.triggerUser.token, currentTask.taskId, toState, newAttributes);

			// the task transitioned in some database. Don't warn about it anymore. In central server mode
			// we may get DCM copies of tasks from multiple databases--we only need to transition the
			// task successfully in one of them (the modifiable_in DB).
			doneTasks[currentTask.taskId] = true;
			delete tasksToErrors[currentTask.taskId];
		}
		catch (error)
		{
			tasksToErrors[currentTask.taskId] =
				change.getMessage('couldNotTransitionTaskXErrorX', [currentTask.taskId, error.message || error.toString()]);
		}
	}

	for (var task in tasksToErrors)
	{
		event.fail(tasksToErrors[task]);
	}
}