// Copyright (c) 2002-2007 Telelogic AB
// All rights reserved.

/*
<trigger>
	<usage>bsf verify_related_change_request_states.js relationship states</usage>
	<version>5.0</version>
	<description>
		CR 遷移中に、関連する変更依頼が指定した状態にあるかどうかを検査します。
		関連する CR の 1 つでも指定した状態にない場合は、このトリガーは失敗します。
		遷移前トリガーとして使用すると、トリガーの失敗によって遷移を取りやめることができます。
	</description>
	<argument1>
		<name>relationship</name>
		<required>yes</required>
		<type>string</type>
		<description>関連 CR を取得するための関係名。
		例えば、親 CR と子 CR の関係付けに "child_problem" 関係を使っているプロセスを使っている場合は、"child_problem" を指定します。
		</description>
	</argument1>
	<argument2>
		<name>states</name>
		<required>yes</required>
		<type>string</type>
		<description>検査すべき状態名 (空白で区切って複数指定可能)。
			関連 CR が指定した状態にない場合は、トリガーは失敗します。</description>
	</argument2>
</trigger>
*/
var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

if (args.length < 2)
{
	log.error('Could not verify related change requests: missing arguments.');
	log.error('Usage: verify_related_change_request_states.js relationship state [or_state...]');
}
else
{
	// Since java array from Rhino is of immutable length, we need to use slice to convert it to a regular javascript array
	var jsArgs = args.slice();
	var relationship = jsArgs.shift();
	var expectedStates = jsArgs;
	var expectedStateSet = new java.util.HashSet(java.util.Arrays.asList(expectedStates));

	var related = change.getRelatedFrom(event.adminUser.token, event.objectId, relationship, ['crstatus']);

	for (var i = 0; i < related.length; i++)
	{
		var currentCr = related[i];
		var actualState = currentCr.getAttribute('crstatus').stringValue;

		if (!expectedStateSet.contains(actualState))
		{
			event.stopTransition(formatError(expectedStates, relationship));
		}
	}
}

function formatError(states, relationship)
{
	if (states.length == 1)
	{
		return change.getMessage('notAllRelationshipXCrsInStateX', [relationship, states[0]]);
	}
	else
	{
		return change.getMessage('notAllRelationshipXCrsInStateListX', [relationship, states.join(', ')]);
	}
}