# <trigger>
#	<usage>perl checkChildAttr.pl reportName attributeName attributeValue</usage>
#	<version>4.4</version>
#	<description>运行父子关系报告，以确保所有子对象都设置了相同的属性值。如果所有子代都具有相同的属性值，那么将允许转变 CR，否则无法转变 CR。
#		该触发器在 ALM 生命周期中使用，用于检查是否可以转变 BCR。运行的报告是 Data_Report_BCR_RCP。会检查所有 RCP，以确定属性 ready_for_final_review 是否设置为 TRUE。
#		确保要检查的属性在报告定义的第二个 [PROBLEM_DEF] 中定义。该触发器通常作为转变前触发器运行。
#   </description>
#	<argument1>
#		<name>reportName</name>
#		<required>true</required>
#		<type></type>
#		<description>要在触发期间运行的报告的名称。</description>
#   </argument1>
#	<argument2>
#		<name>attributeName</name>
#		<required>true</required>
#		<type></type>
#		<description>要检查的属性的名称。</description>
 #      </argument2>
#	<argument3>
#		<name>attributeValue</name>
#		<required>true</required>
#		<type></type>
#		<description>属性的预期值。</description>
#   </argument3>
# </trigger>

use strict;
use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

my $report = shift;
my $attrName = shift;
my $attrValue = shift;

#Create a new TriggerParser object, passing in the location of the trigger xml document.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Create a new csapi object.
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(), $
				      trigger->get_token(), $trigger->get_database());

#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $trigger->get_object_id() . "'";
my $data = $csapi->QueryData($user, $report, $queryString, undef, undef, undef);

#Empty array which will hold the problem numbers for the problems to be transitioned.
my @problemsToTest = ();

#For each top level csapi_cobject_vector_postion.
for(my $i = 0; $i < $data->getDataSize(); $i++)
{
	#Get the child data, this is the associated data, the sub-report data.
	my $childData = $data->getDataObject($i)->getChildData();

	#For each csapi_object_vector_position under the csapi_cobject_vector_assoc tag.
	for(my $j = 0; $j < $childData->getDataSize(); $j++)
	{
		#Get the object vector
		my $objVec = $childData->getDataObject($j);

		# If ready_for_final_review isn't true on this child, then fail.
		if ($objVec->getDataObjectByName($attrName)->getValue() ne "\L$attrValue")
		{
			die "Cannot transition: RCP " . $objVec->getDataObjectByName("problem_number")->getValue() . " is not ready for final review\n";
		}
	}
}

# If we got through all of the childern 
exit(0);